#include <SPI.h>
#include "Font.h"
#include <string.h>
#include "images.h"

const int chipSelectPin = 17; /* Chip Select Pin */
const int DC = 16; /* Data/Command Pin */
const int RST = 18; /* Reset Pin */

void N5110_Cmnd(char DATA) /* Function for sending command to Nokia5110 display */
{
  digitalWrite(DC, LOW); /* DC = 0, display in command mode */
  digitalWrite(chipSelectPin, LOW); /* Make chip select pin low to enable SPI commuincation */
  SPI.transfer(DATA); /* Send data(command) */
  digitalWrite(chipSelectPin, HIGH); /* Make chip select pin high to disable SPI commuincation */
  digitalWrite(DC, HIGH); /* DC = 1, display in data mode */
}

void N5110_Data(char *DATA) /* Function for sending data to Nokia5110 display */
{
  digitalWrite(DC, HIGH); /* DC = 1, display in data mode */
  digitalWrite(chipSelectPin, LOW); /* Make chip select pin low to enable SPI commuincation */
  int len = strlen(DATA);
  for (int g=0; g<len; g++)
  {
    for (int index=0; index<5; index++)
    {
      SPI.transfer(ASCII[DATA[g] - 0x20][index]); /* Send data */
    }
    SPI.transfer(0x00);
  }
  digitalWrite(chipSelectPin, HIGH); /* Make chip select pin high to disable SPI commuincation */
  digitalWrite(DC, LOW); /* DC = 0, display in command mode */
}

void N5110_Reset(void) /* Function for resetting Nokia5110 display */   
{
  digitalWrite(RST, LOW); /* Make reset pin low for resetting the Nokia5110 display */
  delay(100); /* Reset pin needs to be low for at least 100nsec, we are keeping it low for 100msec */
  digitalWrite(RST, HIGH); /* Make reset pin high to bring Nokia5110 display out of reset */
}

void N5110_init(void) /* Function for initializing Nokia5110 display */
{
  N5110_Reset();                    /* reset the display */
  N5110_Cmnd(0x21);                 /* command set in addition mode */
  N5110_Cmnd(0xC0);                 /* set the voltage by sending C0 means VOP = 5V */
  N5110_Cmnd(0x07);                 /* set the temp. coefficient to 3 */
  N5110_Cmnd(0x13);                 /* set value of Voltage Bias System */
  N5110_Cmnd(0x20);                 /* command set in basic mode */
  N5110_Cmnd(0x0C);                 /* display result in normal mode */
}

void lcd_setXY(char x, char y) /* Function for setting cursor to (x,y) co-ordinates */
{
  N5110_Cmnd(x); /* x co-ordinate can be 0x80-0xD3 for position 0-83 */
  N5110_Cmnd(y); /* y co-ordinate can be 0x40-0x45 for position 0-5 */
}

void N5110_clear(void) /* Function for clearing Nokia5110 display */  
{
  digitalWrite(DC, HIGH); /* DC = 1, display in data mode */ 
  digitalWrite(chipSelectPin, LOW); /* Make chip select pin low to enable SPI commuincation */   
  for (int k=0; k<=503; k++)
  {
    SPI.transfer(0x00); /* Send data 0x00 to clear the Nokia5110 display */    
  }  
  digitalWrite(chipSelectPin, HIGH); /* Make chip select pin high to disable SPI commuincation */
  digitalWrite(DC, LOW); /* DC = 0, display in command mode */
}


void N5110_image(const unsigned char *image_data)   /* Function for sending image data to Nokia5110 display */
{
  digitalWrite(DC, HIGH); /* DC = 1, display in data mode */
  digitalWrite(chipSelectPin, LOW); /* Make chip select pin low to enable SPI commuincation */
  for (int k=0; k<504; k++)
  {
    SPI.transfer(image_data[k]); /* Send image data to be displayed */
  }  
  digitalWrite(chipSelectPin, HIGH); /* Make chip select pin high to disable SPI commuincation */  
  digitalWrite(DC, LOW); /* DC = 0, display in command mode */
}


void setup() {
  SPI.begin(); /* Initialize the SPI bus */
  pinMode(chipSelectPin, OUTPUT);
  pinMode(DC, OUTPUT);
  pinMode(RST, OUTPUT);
  delay(1000);
  N5110_init(); /* Initialize the Nokia5110 display */
  delay(100);  
  N5110_clear(); /* Clear the Nokia5110 display */  
  delay(1000);
  lcd_setXY(0x80,0x40);
  N5110_image(Launchpad_Logo);
  delay(3000);
  N5110_clear();
  lcd_setXY(0x9B,0x42);
  N5110_Data("Smiley");
  lcd_setXY(0xA0,0x43);
  N5110_Data("Demo");
  delay(3000);
  lcd_setXY(0x80,0x40);
}

void loop() {  
  N5110_clear();
  N5110_image(Smiley_1);
  delay(300);
  N5110_clear();
  N5110_image(Smiley_2);
  delay(300);
  N5110_clear();
  N5110_image(Smiley_3);
  delay(300);
  N5110_clear();
  N5110_image(Smiley_4);
  delay(300);
}


